//
//  MTLButton.m
//  TravelAdvisor
//
//  Created by narita on 07/10/24.
//  Copyright 2007 MindTools Inc. All rights reserved.
//

#import "MTLButton.h"


@implementation MTLButton

#pragma mark -
#pragma mark class methods

#pragma mark -
#pragma mark init methods

#pragma -
#pragma mark dealloc

#pragma mark -
#pragma mark NSCopying, hash, isEqual:

#pragma mark -
#pragma mark NSCoding methods

#pragma mark -
#pragma mark awakeFromNib-like methods

#pragma mark -
#pragma mark action & canAction methods

#pragma mark -
#pragma mark delegate/datasource methods

- (void)viewDidMoveToWindow
{
    [super viewDidMoveToWindow];
    
    NSWindow* theWindow = [self window];
    
    if( theWindow != nil )
    {
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(updateEnableState)
                                                     name:NSWindowDidUpdateNotification
                                                   object:theWindow];
    }
}

- (void)viewWillMoveToWindow:(NSWindow *)newWindow
{
    [super viewWillMoveToWindow:newWindow];
    
    NSWindow* theWindow = [self window];
    
    if( theWindow != nil )
    {
        [[NSNotificationCenter defaultCenter] removeObserver:self
                                                        name:NSWindowDidUpdateNotification
                                                      object:theWindow];
    }
    
}

#pragma mark -
#pragma mark accessor methods (in pairs)

#pragma mark -
#pragma mark Utility methods

- (void) updateEnableState
{
	// actionが設定されているかをチェック。(target == nil)の場合はfirstResponderだから有効なのでtargetのnilチェックはしない。
    if( [self action] != nil && [self window] != nil && [self isHiddenOrHasHiddenAncestor] == NO)
	{
		id 			theValidator;
		
		// レスポンダーチェーンをたどって実際にactionを処理するオブジェクトを探す。そのオブジェクトがメニューやボタンのenable/disableの値を決定出来るはず
		theValidator = [NSApp targetForAction:[self action]
										   to:[self target]
										 from:self];
		
		
		if([self infoForBinding:@"enabled"])
		{
			// enabledにCocoaBindingが設定されている場合は
			// CocoaBindingの設定を優先させるために何もしない。
		}
		else if ((theValidator == nil) || ![theValidator respondsToSelector:[self action]])
		{
			// actionを実行できるレスポンダーが無い場合は、ボタンは押せない
			[self setEnabled:NO];
		}
		else if ([theValidator respondsToSelector:@selector(validateToolbarItem:)])
		{
			// actionを実行できるレスポンダーが"validateToolbarItem:"を実装しているならば従う
			[self setEnabled:
				[theValidator validateToolbarItem:
					(NSToolbarItem*)self]];
		}
		else if ([theValidator respondsToSelector:@selector(validateUserInterfaceItem:)])
		{
			// actionを実行できるレスポンダーが"validateUserInterfaceItem:"を実装しているならば従う
			[self setEnabled:
				[(id<NSUserInterfaceValidations>)theValidator validateUserInterfaceItem:
					(id<NSValidatedUserInterfaceItem>)self]];
		}
		else
		{
			// レスポンダーがあるけど、"validateXXXX"系のメソッドが無い場合は、ボタンは押せる。
			[self setEnabled:YES];
		}
	}
}

@end
