//
//  NSObject(MTLControlItemEnabler).m
//  TravelAdvisor
//
//  Created by narita on 07/10/15.
//  Copyright 2007 MindTools Inc. All rights reserved.
//

#import "NSObject(MTLControlItemEnabler).h"
#import "MTLRuntimeUtility.h"
#import "NSInvocation(MTLAddition).h"

@implementation NSObject (MTLControlItemEnabler)

- (BOOL) updateEnableUserInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*)inItem
{
    typedef BOOL  (*methodFuncPtr)(id, SEL, id);

    BOOL          theResult   = YES;
    methodFuncPtr theFunction = nil;

    // canメソッドの存在確認
    SEL theSEL = [inItem action];
    if(!theSEL)
    {
        theSEL = [((NSControl*)[(NSCell*)inItem controlView]) action];
    }
    
    SEL		theSelector = prefixedSelector(theSEL, @"can");

    if( [ self respondsToSelector:theSelector ] )
    {
        // canメソッドの実行
        theFunction = (methodFuncPtr)[ self methodForSelector:theSelector];
        theResult   = (*theFunction)(self, theSelector, inItem);
    }
    else if( [[self superclass] conformsToProtocol:@protocol(NSUserInterfaceValidations)] )
    {
        // 親クラスのメソッドを実行
        SEL  theValidateSelector = @selector(validateUserInterfaceItem:);
        
        theFunction = (methodFuncPtr)[ self methodForSelector:theValidateSelector];
        theResult   = (*theFunction)(self, theValidateSelector, inItem);
    }
    
	return theResult;
}


// 値がObject型の物
- (void) updatePropertyName:(NSString*) inName
      userInterfaceFromItem:(NSObject<NSValidatedUserInterfaceItem>*) inItem
{
    // bindingされていない項目のみ実行する
    if([self infoForBinding:inName] == nil)
    {
        SEL theSEL = [inItem action];// view item側のメソッド
        
        if(!theSEL)
        {
            theSEL = [((NSControl*)[(NSCell*)inItem controlView]) action];
        }
        
        SEL theSelector;// control側のメソッド
        theSelector = prefixedSelector(theSEL,
                                       [inName stringByAppendingString:@"Of"]);
        
        if(
           [self respondsToSelector:theSelector]
           &&
           [inItem propertyExistsAtName:inName] & propertyAttributeIsWriteable
           )
        {
            //
            NSInvocation* theInvocation;
            
            theInvocation = [NSInvocation invocationWithMethodSignature:
                             [self methodSignatureForSelector:theSelector]];
            
            [theInvocation setArgument:&inItem
                               atIndex:2];
            [theInvocation setSelector:theSelector];
            [theInvocation invokeWithTarget:self];
            
            [inItem setValue:[theInvocation objectWrappedReturnValue]
                      forKey:inName];            
        }
    }
}

- (BOOL) controlItemEnabler:(NSObject<NSValidatedUserInterfaceItem>*)inItem
{
    BOOL theResult = YES;
    
    [self updatePropertyName:@"state" userInterfaceFromItem:inItem];
    [self updatePropertyName:@"image" userInterfaceFromItem:inItem];
    [self updatePropertyName:@"title" userInterfaceFromItem:inItem];
    [self updatePropertyName:@"label" userInterfaceFromItem:inItem];
    [self updatePropertyName:@"attributedTitle" userInterfaceFromItem:inItem];
    [self updatePropertyName:@"objectValue" userInterfaceFromItem:inItem];
    [self updatePropertyName:@"stringValue" userInterfaceFromItem:inItem];
    [self updatePropertyName:@"attributedStringValue" userInterfaceFromItem:inItem];
    [self updatePropertyName:@"toolTip" userInterfaceFromItem:inItem];

    theResult = [self updateEnableUserInterfaceFromItem:inItem];
    
	return theResult;
}




@end

@implementation NSMenuItem (MTLControlItemEnabler)
- (NSInteger) selectedTag
{
    return [self tag];
}
@end

@implementation NSToolbarItem (MTLControlItemEnabler)
- (NSInteger) selectedTag
{
    return [self tag];
}
@end

@implementation NSCell (MTLControlItemEnabler)
- (NSInteger) selectedTag
{
    return [self tag];
}
@end


